@@ -26,6 +26,7 @@ from firewall.core.prog import runProg
 from firewall.core.logger import log
 from firewall.functions import tempFile, readfile
 from firewall.config import COMMANDS
+import string
 
 PROC_IPxTABLE_NAMES = {
 }
@@ -58,6 +59,11 @@ class ebtables(object):
         self.restore_noflush_option = self._detect_restore_noflush_option()
         self.concurrent_option = self._detect_concurrent_option()
         self.__remove_dangling_lock()
+        self.fill_exists()
+
+    def fill_exists(self):
+        self.command_exists = os.path.exists(self._command)
+        self.restore_command_exists = os.path.exists(self._restore_command)
 
     def __remove_dangling_lock(self):
         if os.path.exists(self.ebtables_lock):
@@ -76,8 +82,8 @@ class ebtables(object):
         # Do not change any rules, just try to use the --concurrent option
         # with -L
         concurrent_option = ""
-        (status, ret) = runProg(self._command, ["--concurrent", "-L"])
-        if status == 0:
+        ret = runProg(self._command, ["--concurrent", "-L"])
+        if ret[0] == 0:
             concurrent_option = "--concurrent"  # concurrent for ebtables lock
 
         return concurrent_option
@@ -121,13 +127,21 @@ class ebtables(object):
                     rule.pop(i)
                     table = rule.pop(i)
 
+            # we can not use joinArgs here, because it would use "'" instead
+            # of '"' for the start and end of the string, this breaks
+            # iptables-restore
+            for i in range(len(rule)):
+                for c in string.whitespace:
+                    if c in rule[i] and not (rule[i].startswith('"') and
+                                             rule[i].endswith('"')):
+                        rule[i] = '"%s"' % rule[i]
+
             table_rules.setdefault(table, []).append(rule)
 
         for table in table_rules:
             temp_file.write("*%s\n" % table)
             for rule in table_rules[table]:
                 temp_file.write(" ".join(rule) + "\n")
-            temp_file.write("COMMIT\n")
 
         temp_file.close()
 
@@ -142,13 +156,10 @@ class ebtables(object):
                                 stdin=temp_file.name)
 
         if log.getDebugLogLevel() > 2:
-            try:
-                readfile(temp_file.name)
-            except:
-                pass
-            else:
+            lines = readfile(temp_file.name)
+            if lines is not None:
                 i = 1
-                for line in readfile(temp_file.name):
+                for line in lines:
                     log.debug3("%8d: %s" % (i, line), nofmt=1, nl=0)
                     if not line.endswith("\n"):
                         log.debug3("", nofmt=1)
@@ -185,9 +196,8 @@ class ebtables(object):
     def used_tables(self):
         return list(BUILT_IN_CHAINS.keys())
 
-    def flush(self, individual=False):
+    def flush(self, transaction=None):
         tables = self.used_tables()
-        rules = [ ]
         for table in tables:
             # Flush firewall rules: -F
             # Delete firewall chains: -X
@@ -198,33 +208,29 @@ class ebtables(object):
                 "-Z": "zero counters",
             }
             for flag in [ "-F", "-X", "-Z" ]:
-                if individual:
+                if transaction is not None:
+                    transaction.add_rule(self.ipv, [ "-t", table, flag ])
+                else:
                     try:
                         self.__run([ "-t", table, flag ])
                     except Exception as msg:
                         log.error("Failed to %s %s: %s",
                                   msgs[flag], self.ipv, msg)
-                else:
-                    rules.append([ "-t", table, flag ])
-        if len(rules) > 0:
-            self.set_rules(rules)
 
-    def set_policy(self, policy, which="used", individual=False):
+    def set_policy(self, policy, which="used", transaction=None):
         if which == "used":
             tables = self.used_tables()
         else:
             tables = list(BUILT_IN_CHAINS.keys())
 
-        rules = [ ]
         for table in tables:
             for chain in BUILT_IN_CHAINS[table]:
-                if individual:
+                if transaction is not None:
+                    transaction.add_rule(self.ipv,
+                                         [ "-t", table, "-P", chain, policy ])
+                else:
                     try:
                         self.__run([ "-t", table, "-P", chain, policy ])
                     except Exception as msg:
                         log.error("Failed to set policy for %s: %s", self.ipv,
                                   msg)
-                else:
-                    rules.append([ "-t", table, "-P", chain, policy ])
-        if len(rules) > 0:
-            self.set_rules(rules)
