@@ -23,6 +23,7 @@ __all__ = [ "Firewall" ]
 
 import os.path, sys
 import copy
+import time
 from firewall import config
 from firewall import functions
 from firewall.core import ipXtables
@@ -36,6 +37,7 @@ from firewall.core.fw_direct import FirewallDirect
 from firewall.core.fw_config import FirewallConfig
 from firewall.core.fw_policies import FirewallPolicies
 from firewall.core.fw_ipset import FirewallIPSet
+from firewall.core.fw_transaction import FirewallTransaction, reverse_rule
 from firewall.core.logger import log
 from firewall.core.io.firewalld_conf import firewalld_conf
 from firewall.core.io.direct import Direct
@@ -121,7 +123,7 @@ class Firewall(object):
 
         if self.ebtables_enabled and \
            "filter" not in self.get_available_tables("eb"):
-            log.error("ebtables not usable, disabling ethernet bridge firewall.")
+            log.warning("ebtables not usable, disabling ethernet bridge firewall.")
             self.ebtables_enabled = False
 
         # is there at least support for ipv4 or ipv6
@@ -131,9 +133,9 @@ class Firewall(object):
 
     def _start_check(self):
         try:
-            x = self.ipset_backend.list()
-        except:
-            log.error("ipset not usable, disabling ipset usage in firewall.")
+            self.ipset_backend.list()
+        except ValueError:
+            log.warning("ipset not usable, disabling ipset usage in firewall.")
             # ipset is not usable, no supported types
             self.ipset_enabled = False
             self.ipset_supported_types = [ ]
@@ -141,6 +143,41 @@ class Firewall(object):
             # ipset is usable, get all supported types
             self.ipset_supported_types = self.ipset_backend.supported_types()
 
+        self.ip4tables_backend.fill_exists()
+        if not self.ip4tables_backend.restore_command_exists:
+            if self.ip4tables_backend.command_exists:
+                log.warning("iptables-restore is missing, using "
+                            "individual calls for IPv4 firewall.")
+            else:
+                log.warning("iptables-restore and iptables are missing, "
+                            "disabling IPv4 firewall.")
+                self.ip4tables_enabled = False
+
+        self.ip6tables_backend.fill_exists()
+        if not self.ip6tables_backend.restore_command_exists:
+            if self.ip6tables_backend.command_exists:
+                log.warning("ip6tables-restore is missing, using "
+                            "individual calls for IPv6 firewall.")
+            else:
+                log.warning("ip6tables-restore and ip6tables are missing, "
+                            "disabling IPv6 firewall.")
+                self.ip6tables_enabled = False
+
+        self.ebtables_backend.fill_exists()
+        if not self.ebtables_backend.restore_command_exists:
+            if self.ebtables_backend.command_exists:
+                log.warning("ebtables-restore is missing, using "
+                            "individual calls for bridge firewall.")
+            else:
+                log.warning("ebtables-restore and ebtables are missing, "
+                            "disabling bridge firewall.")
+                self.ebtables_enabled = False
+
+        if self.ebtables_enabled and not self._individual_calls and \
+           not self.ebtables_backend.restore_noflush_option:
+            log.debug1("ebtables-restore is not supporting the --noflush "
+                       "option, will therefore not be used")
+
     def _start(self, reload=False, complete_reload=False):
         # initialize firewall
         default_zone = config.FALLBACK_ZONE
@@ -199,10 +236,6 @@ class Firewall(object):
                     self._log_denied = value.lower()
                     log.debug1("LogDenied is set to '%s'", self._log_denied)
 
-            if self.ebtables_enabled and not self._individual_calls and \
-               not self.ebtables_backend.restore_noflush_option:
-                log.debug1("ebtables-restore is not supporting the --noflush option, will therefore not be used")
-
         self.config.set_firewalld_conf(copy.deepcopy(self._firewalld_conf))
 
         self._start_check()
@@ -214,10 +247,10 @@ class Firewall(object):
         except Exception as msg:
             if self.policies.query_lockdown():
                 log.error("Failed to load lockdown whitelist '%s': %s",
-                      self.policies.lockdown_whitelist.filename, msg)
+                          self.policies.lockdown_whitelist.filename, msg)
             else:
                 log.debug1("Failed to load lockdown whitelist '%s': %s",
-                      self.policies.lockdown_whitelist.filename, msg)
+                           self.policies.lockdown_whitelist.filename, msg)
 
         # copy policies to config interface
         self.config.set_policies(copy.deepcopy(self.policies))
@@ -285,45 +318,86 @@ class Firewall(object):
         self.direct.set_permanent_config(obj)
         self.config.set_direct(copy.deepcopy(obj))
 
-        if reload:
-            self._set_policy("DROP")
-
         # check if needed tables are there
         self._check_tables()
 
+        if log.getDebugLogLevel() > 0:
+            # get time before flushing and applying
+            tm1 = time.time()
+
+        # Start transaction
+        transaction = FirewallTransaction(self)
+
+        if reload:
+            self.set_policy("DROP", use_transaction=transaction)
+
         # flush rules
-        self._flush()
+        self.flush(use_transaction=transaction)
+
+        # If modules need to be unloaded in complete reload or if there are
+        # ipsets to get applied, limit the transaction to set_policy and flush.
+        #
+        # Future optimization for the ipset case in reload: The transaction
+        # only needs to be split here if there are conflicting ipset types in
+        # exsting ipsets and the configuration in firewalld.
+        if (reload and complete_reload) or \
+           (self.ipset_enabled and self.ipset.has_ipsets()):
+            transaction.execute(True)
+            transaction.clear()
 
         # complete reload: unload modules also
         if reload and complete_reload:
             log.debug1("Unloading firewall modules")
             self.modules_backend.unload_firewall_modules()
 
-        # apply default rules
-        log.debug1("Applying default rule set")
-        self._apply_default_rules()
-
-        # apply settings for loaded ipsets
-        if self.ipset_enabled:
+        # apply settings for loaded ipsets while reloading here
+        if self.ipset_enabled and self.ipset.has_ipsets():
             log.debug1("Applying ipsets")
             self.ipset.apply_ipsets()
 
+        # Start or continue with transaction
+
+        # apply default rules
+        log.debug1("Applying default rule set")
+        self.apply_default_rules(use_transaction=transaction)
+
         # apply settings for loaded zones
         log.debug1("Applying used zones")
-        self.zone.apply_zones()
+        self.zone.apply_zones(use_transaction=transaction)
 
         self._default_zone = self.check_zone(default_zone)
-        self.zone.change_default_zone(None, self._default_zone)
+        self.zone.change_default_zone(None, self._default_zone,
+                                      use_transaction=transaction)
+
+        # Execute transaction
+        transaction.execute(True)
+
+        # Start new transaction for direct rules
+        transaction.clear()
 
         # apply direct chains, rules and passthrough rules
-        log.debug1("Applying direct chains rules and and passthrough rules")
-        self.direct.apply_direct()
+        if self.direct.has_configuration():
+            transaction.enable_generous_mode()
+            log.debug1("Applying direct chains rules and passthrough rules")
+            self.direct.apply_direct(transaction)
+
+            # Execute transaction
+            transaction.execute(True)
+            transaction.disable_generous_mode()
+            transaction.clear()
+
+        del transaction
+
+        if log.getDebugLogLevel() > 1:
+            # get time after flushing and applying
+            tm2 = time.time()
+            log.debug2("Flushing and applying took %f seconds" % (tm2 - tm1))
 
         self._state = "RUNNING"
 
     def start(self):
         self._start()
-        self._set_policy("ACCEPT")
+        self.set_policy("ACCEPT")
 
     def _loader(self, path, reader_type, combine=False):
         # combine: several zone files are getting combined into one obj
@@ -442,7 +516,7 @@ class Firewall(object):
                            orig_obj.filename)
                 try:
                     self.zone.remove_zone(combined_zone.name)
-                except:
+                except Exception:
                     pass
                 self.config.forget_zone(combined_zone.name)
             self.zone.add_zone(combined_zone)
@@ -465,8 +539,8 @@ class Firewall(object):
 
     def stop(self):
         if self.cleanup_on_exit:
-            self._flush()
-            self._set_policy("ACCEPT")
+            self.flush()
+            self.set_policy("ACCEPT")
             self.modules_backend.unload_firewall_modules()
 
         self.cleanup()
@@ -484,105 +558,10 @@ class Firewall(object):
     def del_mark(self, mark):
         self._marks.remove(mark)
 
-    # handle rules, chains and modules
+    # handle modules
 
-    def handle_rules(self, rules, enable, insert=False):
-        if insert:
-            append_delete = { True: "-I", False: "-D", }
-        else:
-            append_delete = { True: "-A", False: "-D", }
-
-        _rules = { }
-        # appends rules
-        # returns None if all worked, else (cleanup rules, error message)
-        for i,value in enumerate(rules):
-            table = chain = None
-            if len(value) == 5:
-                (ipv, table, chain, rule, insert) = value
-                # drop insert rule number if it exists
-                if insert and not enable and isinstance(rule[1], int):
-                    rule.pop(1)
-            elif len(value) == 4:
-                (ipv, table, chain, rule) = value
-                # drop insert rule number if it exists
-                if insert and not enable and isinstance(rule[1], int):
-                    rule.pop(1)
-            elif len(value) == 3:
-                (ipv, rule, insert) = value
-            else:
-                (ipv, rule) = value
-
-            # drop insert rule number if it exists
-            if insert and not enable and isinstance(rule[1], int):
-                rule.pop(1)
-
-            if table and table not in self.get_available_tables(ipv):
-                if ((ipv == "ipv4" and self.ip4tables_enabled) or
-                    (ipv == "ipv6" and self.ip6tables_enabled)):
-                    log.error("Unable to add %s into %s %s" % (rule, ipv,
-                                                               table))
-                continue
-
-            if table is not None:
-                _rule = [ "-t", table, append_delete[enable], ]
-            else:
-                _rule = [ append_delete[enable], ]
-            if chain is not None:
-                _rule.append(chain)
-            _rule += [ "%s" % item for item in rule ]
-
-            if self._individual_calls or \
-               (ipv == "eb" and not
-                self.ebtables_backend.restore_noflush_option):
-                ## run
-                try:
-                    self.rule(ipv, _rule)
-                except Exception as msg:
-                    log.error("Failed to apply rules. A firewall reload might solve the issue if the firewall has been modified using ip*tables or ebtables.")
-                    log.error(msg)
-                    return (rules[:i], msg) # cleanup rules and error message
-            else:
-                _rules.setdefault(ipv, []).append(_rule)
-
-        try:
-            for ipv in _rules:
-                self.rules(ipv, _rules[ipv])
-        except Exception as msg:
-            log.error("Failed to apply rules. A firewall reload might solve the issue if the firewall has been modified using ip*tables or ebtables.")
-            log.error(msg)
-            return ([ ], msg) # no cleanup rules and error message
-
-        return None
-
-    def handle_chains(self, rules, enable):
-        new_delete = { True: "-N", False: "-X" }
-
-        _rules = { }
-        # appends chains
-        # returns None if all worked, else (cleanup chains, error message)
-        for i,(ipv, rule) in enumerate(rules):
-            _rule = [ new_delete[enable], ] + rule
-            if self._individual_calls or \
-               (ipv == "eb" and not
-                self.ebtables_backend.restore_noflush_option):
-                try:
-                    self.rule(ipv, _rule)
-                except Exception as msg:
-                    log.error(msg)
-                    return (rules[:i], msg) # cleanup chains and error message
-            else:
-                _rules.setdefault(ipv, []).append(_rule)
-        try:
-            for ipv in _rules:
-                self.rules(ipv, _rules[ipv])
-        except Exception as msg:
-            log.error("Failed to apply rules. A firewall reload might solve the issue if the firewall has been modified using ip*tables or ebtables.")
-            log.error(msg)
-            return ([ ], msg) # no cleanup rules and error message
-        return None
-
-    def handle_modules(self, modules, enable):
-        for i,module in enumerate(modules):
+    def handle_modules(self, _modules, enable):
+        for i,module in enumerate(_modules):
             if enable:
                 (status, msg) = self.modules_backend.load_module(module)
             else:
@@ -592,7 +571,7 @@ class Firewall(object):
                     (status, msg) = self.modules_backend.unload_module(module)
             if status != 0:
                 if enable:
-                    return (modules[:i], msg) # cleanup modules and error msg
+                    return (_modules[:i], msg) # cleanup modules and error msg
                 # else: ignore cleanup
 
             if enable:
@@ -606,7 +585,7 @@ class Firewall(object):
         return None
 
     # apply default rules
-    def __apply_default_rules(self, ipv):
+    def __apply_default_rules(self, ipv, transaction):
         default_rules = { }
 
         if ipv in [ "ipv4", "ipv6" ]:
@@ -620,7 +599,6 @@ class Firewall(object):
             for table in x.LOG_RULES:
                 default_rules.setdefault(table, []).extend(x.LOG_RULES[table])
 
-        rules = { }
         for table in default_rules:
             if table not in self.get_available_tables(ipv):
                 continue
@@ -630,99 +608,109 @@ class Firewall(object):
                     _rule = prefix + rule
                 else:
                     _rule = prefix + functions.splitArgs(rule)
-                if self._individual_calls or \
-                   (ipv == "eb" and not
-                    self.ebtables_backend.restore_noflush_option):
-                    self.rule(ipv, _rule)
-                else:
-                    rules.setdefault(ipv, []).append(_rule)
+                #if self._individual_calls or \
+                #   (ipv == "eb" and not
+                #    self.ebtables_backend.restore_noflush_option):
+                #    self.rule(ipv, _rule)
+                #else:
+                #    transaction.add_rule(ipv, _rule)
+                transaction.add_rule(ipv, _rule)
 
-        for ipv in rules:
-            self.rules(ipv, rules[ipv])
+    def apply_default_rules(self, use_transaction=None):
+        if use_transaction is None:
+            transaction = FirewallTransaction(self)
+        else:
+            transaction = use_transaction
 
-    def _apply_default_rules(self):
         for ipv in [ "ipv4", "ipv6", "eb" ]:
-            self.__apply_default_rules(ipv)
+            self.__apply_default_rules(ipv, transaction)
 
         if self.ipv6_rpfilter_enabled and \
            "raw" in self.get_available_tables("ipv6"):
             # here is no check for ebtables.restore_noflush_option needed
             # as ebtables is not used in here
-            rules = [
-                ("ipv6", [ "PREROUTING", 1, "-t", "raw",
-                           "-p", "ipv6-icmp",
-                           "--icmpv6-type=router-advertisement",
-                           "-j", "ACCEPT" ]), # RHBZ#1058505
-                ("ipv6", [ "PREROUTING", 2, "-t", "raw",
-                           "-m", "rpfilter", "--invert", "-j", "DROP" ]),
-            ]
+            transaction.add_rule("ipv6",
+                                 [ "-I", "PREROUTING", "1", "-t", "raw",
+                                   "-p", "ipv6-icmp",
+                                   "--icmpv6-type=router-advertisement",
+                                   "-j", "ACCEPT" ]) # RHBZ#1058505
+            transaction.add_rule("ipv6",
+                                 [ "-I", "PREROUTING", "2", "-t", "raw",
+                                   "-m", "rpfilter", "--invert", "-j", "DROP" ])
             if self._log_denied != "off":
-                rules.append(("ipv6", [ "PREROUTING", 2, "-t", "raw",
-                                        "-m", "rpfilter", "--invert",
-                                        "-j", "LOG",
-                                        "--log-prefix", "rpfilter_DROP: " ]))
-            # handle rules
-            ret = self.handle_rules(rules, True, insert=True)
-            if ret:
-                (cleanup_rules, msg) = ret
-                self.handle_rules(cleanup_rules, False)
-                log.error(msg)
+                transaction.add_rule("ipv6",
+                                     [ "-I", "PREROUTING", "2", "-t", "raw",
+                                       "-m", "rpfilter", "--invert",
+                                       "-j", "LOG",
+                                       "--log-prefix", "rpfilter_DROP: " ])
+
+        if use_transaction is None:
+            transaction.execute(True)
 
     # flush and policy
 
-    def _flush(self):
+    def flush(self, use_transaction=None):
+        if use_transaction is None:
+            transaction = FirewallTransaction(self)
+        else:
+            transaction = use_transaction
+
         log.debug1("Flushing rule set")
 
         if self.ip4tables_enabled:
             try:
-                self.ip4tables_backend.flush(individual=self._individual_calls)
+                self.ip4tables_backend.flush(transaction)
             except Exception as e:
                 log.error("Failed to flush ipv4 firewall: %s" % e)
         if self.ip6tables_enabled:
             try:
-                self.ip6tables_backend.flush(individual=self._individual_calls)
+                self.ip6tables_backend.flush(transaction)
             except Exception as e:
                 log.error("Failed to flush ipv6 firewall: %s" % e)
         if self.ebtables_enabled:
             try:
-                self.ebtables_backend.flush(
-                    individual=self._individual_calls or \
-                    not self.ebtables_backend.restore_noflush_option)
+                self.ebtables_backend.flush(transaction)
             except Exception as e:
                 log.error("Failed to flush eb firewall: %s" % e)
 
-    def _set_policy(self, policy, which="used"):
+        if use_transaction is None:
+            transaction.execute(True)
+
+    def set_policy(self, policy, which="used", use_transaction=None):
+        if use_transaction is None:
+            transaction = FirewallTransaction(self)
+        else:
+            transaction = use_transaction
+
         log.debug1("Setting policy to '%s'", policy)
 
         if self.ip4tables_enabled:
             try:
-                self.ip4tables_backend.set_policy(
-                    policy, which, individual=self._individual_calls)
+                self.ip4tables_backend.set_policy(policy, which, transaction)
             except Exception as e:
                 log.error("Failed to set policy of ipv4 firewall: %s" % e)
 
         if self.ip6tables_enabled:
             try:
-                self.ip6tables_backend.set_policy(
-                    policy, which, individual=self._individual_calls)
+                self.ip6tables_backend.set_policy(policy, which, transaction)
             except Exception as e:
                 log.error("Failed to set policy of ipv6 firewall: %s" % e)
         if self.ebtables_enabled:
             try:
-                self.ebtables_backend.set_policy(
-                    policy, which,
-                    individual=self._individual_calls or \
-                    not self.ebtables_backend.restore_noflush_option)
+                self.ebtables_backend.set_policy(policy, which, transaction)
             except Exception as e:
                 log.error("Failed to set policy of eb firewall: %s" % e)
 
+        if use_transaction is None:
+            transaction.execute(True)
+
     # rule function used in handle_ functions
 
     def rule(self, ipv, rule):
         # replace %%REJECT%%
         try:
             i = rule.index("%%REJECT%%")
-        except:
+        except ValueError:
             pass
         else:
             if ipv in [ "ipv4", "ipv6" ]:
@@ -735,7 +723,7 @@ class Firewall(object):
         # replace %%ICMP%%
         try:
             i = rule.index("%%ICMP%%")
-        except:
+        except ValueError:
             pass
         else:
             if ipv in [ "ipv4", "ipv6" ]:
@@ -747,7 +735,7 @@ class Firewall(object):
         # replace %%LOGTYPE%%
         try:
             i = rule.index("%%LOGTYPE%%")
-        except:
+        except ValueError:
             pass
         else:
             if self._log_denied == "off":
@@ -794,7 +782,7 @@ class Firewall(object):
             # replace %%REJECT%%
             try:
                 i = rule.index("%%REJECT%%")
-            except:
+            except ValueError:
                 pass
             else:
                 if ipv in [ "ipv4", "ipv6" ]:
@@ -807,7 +795,7 @@ class Firewall(object):
             # replace %%ICMP%%
             try:
                 i = rule.index("%%ICMP%%")
-            except:
+            except ValueError:
                 pass
             else:
                 if ipv in [ "ipv4", "ipv6" ]:
@@ -819,14 +807,14 @@ class Firewall(object):
             # replace %%LOGTYPE%%
             try:
                 i = rule.index("%%LOGTYPE%%")
-            except:
+            except ValueError:
                 pass
             else:
                 if self._log_denied == "off":
                     continue
                 if ipv not in [ "ipv4", "ipv6" ]:
                     raise FirewallError(errors.INVALID_IPV,
-                                    "'%s' not in {'ipv4'|'ipv6'}" % ipv)
+                                        "'%s' not in {'ipv4'|'ipv6'}" % ipv)
                 if self._log_denied in [ "unicast", "broadcast",
                                          "multicast" ]:
                     rule[i:i+1] = [ "-m", "pkttype", "--pkt-type",
@@ -836,23 +824,53 @@ class Firewall(object):
 
             _rules.append(rule)
 
+        backend = None
         if ipv == "ipv4":
-            # do not call if disabled
             if self.ip4tables_enabled:
-                return self.ip4tables_backend.set_rules(_rules)
+                # do not call if disabled
+                backend = self.ip4tables_backend
         elif ipv == "ipv6":
-            # do not call if disabled
             if self.ip6tables_enabled:
-                return self.ip6tables_backend.set_rules(_rules)
+                # do not call if disabled
+                backend = self.ip6tables_backend
         elif ipv == "eb":
-            # do not call if disabled
             if self.ebtables_enabled:
-                return self.ebtables_backend.set_rules(_rules)
+                # do not call if disabled
+                backend = self.ebtables_backend
         else:
             raise FirewallError(errors.INVALID_IPV,
                                 "'%s' not in {'ipv4'|'ipv6'|'eb'}" % ipv)
 
-        return ""
+        if not backend:
+            return ""
+
+        if self._individual_calls or \
+           not backend.restore_command_exists or \
+           (ipv == "eb" and not self.ebtables_backend.restore_noflush_option):
+            for i,rule in enumerate(_rules):
+                # remove leading and trailing '"' for use with execve
+                j = 0
+                while j < len(rule):
+                    x = rule[j]
+                    if len(x) > 2 and x[0] == '"' and x[-1] == '"':
+                        rule[j] = x[1:-1]
+                    j += 1
+
+                try:
+                    backend.set_rule(rule)
+                except Exception as msg:
+                    log.error("Failed to apply rules. A firewall reload might solve the issue if the firewall has been modified using ip*tables or ebtables.")
+                    log.error(msg)
+                    for rule in reversed(_rules[:i]):
+                        try:
+                            backend.set_rule(reverse_rule(rule))
+                        except Exception:
+                            # ignore errors here
+                            pass
+                    return False
+            return True
+        else:
+            return backend.set_rules(_rules)
 
     # check functions
 
@@ -876,24 +894,13 @@ class Firewall(object):
         self.service.check_service(service)
 
     def check_port(self, port):
-        range = functions.getPortRange(port)
-
-        if range == -2 or range == -1 or range is None or \
-                (len(range) == 2 and range[0] >= range[1]):
-            if range == -2:
-                log.debug1("'%s': port > 65535" % port)
-            elif range == -1:
-                log.debug1("'%s': port is invalid" % port)
-            elif range is None:
-                log.debug1("'%s': port is ambiguous" % port)
-            elif len(range) == 2 and range[0] >= range[1]:
-                log.debug1("'%s': range start >= end" % port)
+        if not functions.check_port(port):
             raise FirewallError(errors.INVALID_PORT, port)
 
     def check_tcpudp(self, protocol):
         if not protocol:
             raise FirewallError(errors.MISSING_PROTOCOL)
-        if not protocol in [ "tcp", "udp" ]:
+        if protocol not in [ "tcp", "udp" ]:
             raise FirewallError(errors.INVALID_PROTOCOL,
                                 "'%s' not in {'tcp'|'udp'}" % protocol)
 
@@ -941,7 +948,7 @@ class Firewall(object):
         # start
         self._start(reload=True, complete_reload=stop)
 
-        # handle interfaces in the default zone and move them to the new 
+        # handle interfaces in the default zone and move them to the new
         # default zone if it changed
         _new_dz = self.get_default_zone()
         if _new_dz != _old_dz:
@@ -949,12 +956,12 @@ class Firewall(object):
             # https://github.com/t-woerner/firewalld/issues/53
             if _new_dz not in _zone_interfaces:
                 _zone_interfaces[_new_dz] = { }
-            # default zone changed. Move interfaces from old default zone to 
+            # default zone changed. Move interfaces from old default zone to
             # the new one.
             for iface, settings in list(_zone_interfaces[_old_dz].items()):
                 if settings["__default__"]:
                     # move only those that were added to default zone
-                    # (not those that were added to specific zone same as 
+                    # (not those that were added to specific zone same as
                     # default)
                     _zone_interfaces[_new_dz][iface] = \
                         _zone_interfaces[_old_dz][iface]
@@ -964,7 +971,7 @@ class Firewall(object):
         for zone in self.zone.get_zones():
             if zone in _zone_interfaces:
                 self.zone.set_settings(zone, { "interfaces":
-                                                   _zone_interfaces[zone] })
+                                               _zone_interfaces[zone] })
                 del _zone_interfaces[zone]
             else:
                 log.info1("New zone '%s'.", zone)
@@ -977,12 +984,12 @@ class Firewall(object):
         # restore direct config
         self.direct.set_config(_direct_config)
 
-        # enable panic mode again if it has been enabled before or set policy 
+        # enable panic mode again if it has been enabled before or set policy
         # to ACCEPT
         if _panic:
             self.enable_panic_mode()
         else:
-            self._set_policy("ACCEPT")
+            self.set_policy("ACCEPT")
 
     # STATE
 
@@ -998,7 +1005,7 @@ class Firewall(object):
 
         # TODO: use rule in raw table not default chain policy
         try:
-            self._set_policy("DROP", "all")
+            self.set_policy("DROP", "all")
         except Exception as msg:
             raise FirewallError(errors.COMMAND_FAILED, msg)
         self._panic = True
@@ -1010,7 +1017,7 @@ class Firewall(object):
 
         # TODO: use rule in raw table not default chain policy
         try:
-            self._set_policy("ACCEPT", "all")
+            self.set_policy("ACCEPT", "all")
         except Exception as msg:
             raise FirewallError(errors.COMMAND_FAILED, msg)
         self._panic = False
