@@ -33,7 +33,8 @@ from firewall import config
 from firewall.core.base import DEFAULT_ZONE_TARGET
 from firewall.core.watcher import Watcher
 from firewall.core.logger import log
-from firewall.server.decorators import *
+from firewall.server.decorators import handle_exceptions, \
+    dbus_handle_exceptions, dbus_service_method
 from firewall.server.config_icmptype import FirewallDConfigIcmpType
 from firewall.server.config_service import FirewallDConfigService
 from firewall.server.config_zone import FirewallDConfigZone
@@ -81,10 +82,12 @@ class FirewallDConfig(slip.dbus.service.Object):
         self.watcher.add_watch_dir(config.ETC_FIREWALLD_SERVICES)
         self.watcher.add_watch_dir(config.FIREWALLD_ZONES)
         self.watcher.add_watch_dir(config.ETC_FIREWALLD_ZONES)
-        for filename in sorted(os.listdir(config.ETC_FIREWALLD_ZONES)):
-            path = "%s/%s" % (config.ETC_FIREWALLD_ZONES, filename)
-            if os.path.isdir(path):
-                self.watcher.add_watch_dir(path)
+        # Add watches for combined zone directories
+        if os.path.exists(config.ETC_FIREWALLD_ZONES):
+            for filename in sorted(os.listdir(config.ETC_FIREWALLD_ZONES)):
+                path = "%s/%s" % (config.ETC_FIREWALLD_ZONES, filename)
+                if os.path.isdir(path):
+                    self.watcher.add_watch_dir(path)
         self.watcher.add_watch_file(config.LOCKDOWN_WHITELIST)
         self.watcher.add_watch_file(config.FIREWALLD_DIRECT)
         self.watcher.add_watch_file(config.FIREWALLD_CONF)
@@ -94,7 +97,8 @@ class FirewallDConfig(slip.dbus.service.Object):
                                               { "CleanupOnExit": "readwrite",
                                                 "IPv6_rpfilter": "readwrite",
                                                 "Lockdown": "readwrite",
-                                                "MinimalMark": "readwrite" })
+                                                "MinimalMark": "readwrite",
+                                                "IndividualCalls": "readwrite"})
 
     @handle_exceptions
     def _init_vars(self):
@@ -123,21 +127,21 @@ class FirewallDConfig(slip.dbus.service.Object):
     @handle_exceptions
     def reload(self):
         while len(self.ipsets) > 0:
-            x = self.ipsets.pop()
-            x.unregister()
-            del x
+            item = self.ipsets.pop()
+            item.unregister()
+            del item
         while len(self.icmptypes) > 0:
-            x = self.icmptypes.pop()
-            x.unregister()
-            del x
+            item = self.icmptypes.pop()
+            item.unregister()
+            del item
         while len(self.services) > 0:
-            x = self.services.pop()
-            x.unregister()
-            del x
+            item = self.services.pop()
+            item.unregister()
+            del item
         while len(self.zones) > 0:
-            x = self.zones.pop()
-            x.unregister()
-            del x
+            item = self.zones.pop()
+            item.unregister()
+            del item
         self._init_vars()
 
     @handle_exceptions
@@ -146,7 +150,12 @@ class FirewallDConfig(slip.dbus.service.Object):
             old_props = self.GetAll(config.dbus.DBUS_INTERFACE_CONFIG)
             log.debug1("config: Reloading firewalld config file '%s'",
                        config.FIREWALLD_CONF)
-            self.config.update_firewalld_conf()
+            try:
+                self.config.update_firewalld_conf()
+            except Exception as msg:
+                log.error("Failed to load firewalld.conf file '%s': %s" % \
+                          (name, msg))
+                return
             props = self.GetAll(config.dbus.DBUS_INTERFACE_CONFIG)
             for key in props.keys():
                 if key in old_props and \
@@ -160,7 +169,11 @@ class FirewallDConfig(slip.dbus.service.Object):
         if (name.startswith(config.FIREWALLD_ICMPTYPES) or \
             name.startswith(config.ETC_FIREWALLD_ICMPTYPES)) and \
            name.endswith(".xml"):
-            (what, obj) = self.config.update_icmptype_from_path(name)
+            try:
+                (what, obj) = self.config.update_icmptype_from_path(name)
+            except Exception as msg:
+                log.error("Failed to load icmptype file '%s': %s" % (name, msg))
+                return
             if what == "new":
                 self._addIcmpType(obj)
             elif what == "remove":
@@ -171,7 +184,11 @@ class FirewallDConfig(slip.dbus.service.Object):
         elif (name.startswith(config.FIREWALLD_SERVICES) or \
               name.startswith(config.ETC_FIREWALLD_SERVICES)) and \
              name.endswith(".xml"):
-            (what, obj) = self.config.update_service_from_path(name)
+            try:
+                (what, obj) = self.config.update_service_from_path(name)
+            except Exception as msg:
+                log.error("Failed to load service file '%s': %s" % (name, msg))
+                return
             if what == "new":
                 self._addService(obj)
             elif what == "remove":
@@ -179,10 +196,14 @@ class FirewallDConfig(slip.dbus.service.Object):
             elif what == "update":
                 self._updateService(obj)
 
-        elif (name.startswith(config.FIREWALLD_ZONES) or \
-              name.startswith(config.ETC_FIREWALLD_ZONES)):
+        elif name.startswith(config.FIREWALLD_ZONES) or \
+             name.startswith(config.ETC_FIREWALLD_ZONES):
             if name.endswith(".xml"):
-                (what, obj) = self.config.update_zone_from_path(name)
+                try:
+                    (what, obj) = self.config.update_zone_from_path(name)
+                except Exception as msg:
+                    log.error("Failed to load zone file '%s': %s" % (name, msg))
+                    return
                 if what == "new":
                     self._addZone(obj)
                 elif what == "remove":
@@ -191,8 +212,8 @@ class FirewallDConfig(slip.dbus.service.Object):
                     self._updateZone(obj)
             elif name.startswith(config.ETC_FIREWALLD_ZONES):
                 # possible combined zone base directory
-                x = name.replace(config.ETC_FIREWALLD_ZONES, "").strip("/")
-                if len(x) < 1 or "/" in x:
+                _name = name.replace(config.ETC_FIREWALLD_ZONES, "").strip("/")
+                if len(_name) < 1 or "/" in _name:
                     # if there is a / in x, then it is a sub sub directory
                     # ignore it
                     return
@@ -205,7 +226,13 @@ class FirewallDConfig(slip.dbus.service.Object):
         elif (name.startswith(config.FIREWALLD_IPSETS) or \
               name.startswith(config.ETC_FIREWALLD_IPSETS)) and \
              name.endswith(".xml"):
-            (what, obj) = self.config.update_ipset_from_path(name)
+            try:
+                (what, obj) = self.config.update_ipset_from_path(name)
+            except Exception as msg:
+                log.error("Failed to load ipset file '%s': %s" % (name,
+                                                                  msg))
+
+                return
             if what == "new":
                 self._addIPSet(obj)
             elif what == "remove":
@@ -214,11 +241,21 @@ class FirewallDConfig(slip.dbus.service.Object):
                 self._updateIPSet(obj)
 
         elif name == config.LOCKDOWN_WHITELIST:
-            self.config.update_lockdown_whitelist()
+            try:
+                self.config.update_lockdown_whitelist()
+            except Exception as msg:
+                log.error("Failed to load lockdown whitelist file '%s': %s" % \
+                          (name, msg))
+                return
             self.LockdownWhitelistUpdated()
 
         elif name == config.FIREWALLD_DIRECT:
-            self.config.update_direct()
+            try:
+                self.config.update_direct()
+            except Exception as msg:
+                log.error("Failed to load direct rules file '%s': %s" % (name,
+                                                                         msg))
+                return
             self.Updated()
 
 
@@ -448,7 +485,7 @@ class FirewallDConfig(slip.dbus.service.Object):
     @dbus_service_method(dbus.PROPERTIES_IFACE, in_signature='ss',
                          out_signature='v')
     @dbus_handle_exceptions
-    def Get(self, interface_name, property_name, sender=None):
+    def Get(self, interface_name, property_name, sender=None): # pylint: disable=W0613
         # get a property
         interface_name = dbus_to_python(interface_name, str)
         property_name = dbus_to_python(property_name, str)
@@ -464,7 +501,7 @@ class FirewallDConfig(slip.dbus.service.Object):
     @dbus_service_method(dbus.PROPERTIES_IFACE, in_signature='s',
                          out_signature='a{sv}')
     @dbus_handle_exceptions
-    def GetAll(self, interface_name, sender=None):
+    def GetAll(self, interface_name, sender=None): # pylint: disable=W0613
         interface_name = dbus_to_python(interface_name, str)
         log.debug1("config.GetAll('%s')", interface_name)
 
@@ -496,7 +533,7 @@ class FirewallDConfig(slip.dbus.service.Object):
                 "FirewallD does not implement %s" % interface_name)
 
         if property_name in [ "MinimalMark", "CleanupOnExit", "Lockdown",
-                              "IPv6_rpfilter" ]:
+                              "IPv6_rpfilter", "IndividualCalls" ]:
             if property_name == "MinimalMark":
                 try:
                     int(new_value)
@@ -508,7 +545,7 @@ class FirewallDConfig(slip.dbus.service.Object):
                 raise FirewallError(errors.INVALID_VALUE, "'%s' for %s" % \
                                             (new_value, property_name))
             if property_name in [ "CleanupOnExit", "Lockdown",
-                                  "IPv6_rpfilter" ]:
+                                  "IPv6_rpfilter", "IndividualCalls" ]:
                 if new_value.lower() not in [ "yes", "no", "true", "false" ]:
                     raise FirewallError(errors.INVALID_VALUE, "'%s' for %s" % \
                                             (new_value, property_name))
@@ -537,7 +574,7 @@ class FirewallDConfig(slip.dbus.service.Object):
     @slip.dbus.polkit.require_auth(config.dbus.PK_ACTION_INFO)
     @dbus_service_method(dbus.INTROSPECTABLE_IFACE, out_signature='s')
     @dbus_handle_exceptions
-    def Introspect(self, sender=None):
+    def Introspect(self, sender=None): # pylint: disable=W0613
         log.debug2("config.Introspect()")
 
         data = super(FirewallDConfig, self).Introspect(self.path,
@@ -552,14 +589,14 @@ class FirewallDConfig(slip.dbus.service.Object):
     @dbus_service_method(config.dbus.DBUS_INTERFACE_CONFIG_POLICIES,
                          out_signature=LockdownWhitelist.DBUS_SIGNATURE)
     @dbus_handle_exceptions
-    def getLockdownWhitelist(self, sender=None):
+    def getLockdownWhitelist(self, sender=None): # pylint: disable=W0613
         log.debug1("config.policies.getLockdownWhitelist()")
         return self.config.get_policies().lockdown_whitelist.export_config()
 
-    @dbus_service_method(config.dbus.DBUS_INTERFACE_CONFIG_POLICIES, 
+    @dbus_service_method(config.dbus.DBUS_INTERFACE_CONFIG_POLICIES,
                          in_signature=LockdownWhitelist.DBUS_SIGNATURE)
     @dbus_handle_exceptions
-    def setLockdownWhitelist(self, settings, sender=None):
+    def setLockdownWhitelist(self, settings, sender=None): # pylint: disable=W0613
         log.debug1("config.policies.setLockdownWhitelist(...)")
         settings = dbus_to_python(settings)
         self.config.get_policies().lockdown_whitelist.import_config(settings)
@@ -603,7 +640,7 @@ class FirewallDConfig(slip.dbus.service.Object):
     @dbus_service_method(config.dbus.DBUS_INTERFACE_CONFIG_POLICIES,
                          in_signature='s', out_signature='b')
     @dbus_handle_exceptions
-    def queryLockdownWhitelistCommand(self, command, sender=None):
+    def queryLockdownWhitelistCommand(self, command, sender=None): # pylint: disable=W0613
         command = dbus_to_python(command)
         log.debug1("config.policies.queryLockdownWhitelistCommand('%s')",
                    command)
@@ -612,7 +649,7 @@ class FirewallDConfig(slip.dbus.service.Object):
     @dbus_service_method(config.dbus.DBUS_INTERFACE_CONFIG_POLICIES,
                          out_signature='as')
     @dbus_handle_exceptions
-    def getLockdownWhitelistCommands(self, sender=None):
+    def getLockdownWhitelistCommands(self, sender=None): # pylint: disable=W0613
         log.debug1("config.policies.getLockdownWhitelistCommands()")
         return self.getLockdownWhitelist()[0]
 
@@ -648,7 +685,7 @@ class FirewallDConfig(slip.dbus.service.Object):
     @dbus_service_method(config.dbus.DBUS_INTERFACE_CONFIG_POLICIES,
                          in_signature='s', out_signature='b')
     @dbus_handle_exceptions
-    def queryLockdownWhitelistContext(self, context, sender=None):
+    def queryLockdownWhitelistContext(self, context, sender=None): # pylint: disable=W0613
         context = dbus_to_python(context)
         log.debug1("config.policies.queryLockdownWhitelistContext('%s')",
                    context)
@@ -657,7 +694,7 @@ class FirewallDConfig(slip.dbus.service.Object):
     @dbus_service_method(config.dbus.DBUS_INTERFACE_CONFIG_POLICIES,
                          out_signature='as')
     @dbus_handle_exceptions
-    def getLockdownWhitelistContexts(self, sender=None):
+    def getLockdownWhitelistContexts(self, sender=None): # pylint: disable=W0613
         log.debug1("config.policies.getLockdownWhitelistContexts()")
         return self.getLockdownWhitelist()[1]
 
@@ -692,7 +729,7 @@ class FirewallDConfig(slip.dbus.service.Object):
     @dbus_service_method(config.dbus.DBUS_INTERFACE_CONFIG_POLICIES,
                          in_signature='s', out_signature='b')
     @dbus_handle_exceptions
-    def queryLockdownWhitelistUser(self, user, sender=None):
+    def queryLockdownWhitelistUser(self, user, sender=None): # pylint: disable=W0613
         user = dbus_to_python(user)
         log.debug1("config.policies.queryLockdownWhitelistUser('%s')", user)
         return user in self.getLockdownWhitelist()[2]
@@ -700,7 +737,7 @@ class FirewallDConfig(slip.dbus.service.Object):
     @dbus_service_method(config.dbus.DBUS_INTERFACE_CONFIG_POLICIES,
                          out_signature='as')
     @dbus_handle_exceptions
-    def getLockdownWhitelistUsers(self, sender=None):
+    def getLockdownWhitelistUsers(self, sender=None): # pylint: disable=W0613
         log.debug1("config.policies.getLockdownWhitelistUsers()")
         return self.getLockdownWhitelist()[2]
 
@@ -735,7 +772,7 @@ class FirewallDConfig(slip.dbus.service.Object):
     @dbus_service_method(config.dbus.DBUS_INTERFACE_CONFIG_POLICIES,
                          in_signature='i', out_signature='b')
     @dbus_handle_exceptions
-    def queryLockdownWhitelistUid(self, uid, sender=None):
+    def queryLockdownWhitelistUid(self, uid, sender=None): # pylint: disable=W0613
         uid = dbus_to_python(uid)
         log.debug1("config.policies.queryLockdownWhitelistUid(%d)", uid)
         return uid in self.getLockdownWhitelist()[3]
@@ -743,7 +780,7 @@ class FirewallDConfig(slip.dbus.service.Object):
     @dbus_service_method(config.dbus.DBUS_INTERFACE_CONFIG_POLICIES,
                          out_signature='ai')
     @dbus_handle_exceptions
-    def getLockdownWhitelistUids(self, sender=None):
+    def getLockdownWhitelistUids(self, sender=None): # pylint: disable=W0613
         log.debug1("config.policies.getLockdownWhitelistUids()")
         return self.getLockdownWhitelist()[3]
 
@@ -753,16 +790,27 @@ class FirewallDConfig(slip.dbus.service.Object):
 
     @dbus_service_method(config.dbus.DBUS_INTERFACE_CONFIG, out_signature='ao')
     @dbus_handle_exceptions
-    def listIPSets(self, sender=None):
+    def listIPSets(self, sender=None): # pylint: disable=W0613
         """list ipsets objects paths
         """
         log.debug1("config.listIPSets()")
         return self.ipsets
 
+    @dbus_service_method(config.dbus.DBUS_INTERFACE_CONFIG, out_signature='as')
+    @dbus_handle_exceptions
+    def getIPSetNames(self, sender=None): # pylint: disable=W0613
+        """get ipset names
+        """
+        log.debug1("config.getIPSetNames()")
+        ipsets = [ ]
+        for obj in self.ipsets:
+            ipsets.append(obj.obj.name)
+        return sorted(ipsets)
+
     @dbus_service_method(config.dbus.DBUS_INTERFACE_CONFIG, in_signature='s',
                          out_signature='o')
     @dbus_handle_exceptions
-    def getIPSetByName(self, ipset, sender=None):
+    def getIPSetByName(self, ipset, sender=None): # pylint: disable=W0613
         """object path of ipset with given name
         """
         ipset = dbus_to_python(ipset, str)
@@ -797,16 +845,27 @@ class FirewallDConfig(slip.dbus.service.Object):
 
     @dbus_service_method(config.dbus.DBUS_INTERFACE_CONFIG, out_signature='ao')
     @dbus_handle_exceptions
-    def listIcmpTypes(self, sender=None):
+    def listIcmpTypes(self, sender=None): # pylint: disable=W0613
         """list icmptypes objects paths
         """
         log.debug1("config.listIcmpTypes()")
         return self.icmptypes
 
+    @dbus_service_method(config.dbus.DBUS_INTERFACE_CONFIG, out_signature='as')
+    @dbus_handle_exceptions
+    def getIcmpTypeNames(self, sender=None): # pylint: disable=W0613
+        """get icmptype names
+        """
+        log.debug1("config.getIcmpTypeNames()")
+        icmptypes = [ ]
+        for obj in self.icmptypes:
+            icmptypes.append(obj.obj.name)
+        return sorted(icmptypes)
+
     @dbus_service_method(config.dbus.DBUS_INTERFACE_CONFIG, in_signature='s',
                          out_signature='o')
     @dbus_handle_exceptions
-    def getIcmpTypeByName(self, icmptype, sender=None):
+    def getIcmpTypeByName(self, icmptype, sender=None): # pylint: disable=W0613
         """object path of icmptype with given name
         """
         icmptype = dbus_to_python(icmptype, str)
@@ -840,16 +899,27 @@ class FirewallDConfig(slip.dbus.service.Object):
 
     @dbus_service_method(config.dbus.DBUS_INTERFACE_CONFIG, out_signature='ao')
     @dbus_handle_exceptions
-    def listServices(self, sender=None):
+    def listServices(self, sender=None): # pylint: disable=W0613
         """list services objects paths
         """
         log.debug1("config.listServices()")
         return self.services
 
+    @dbus_service_method(config.dbus.DBUS_INTERFACE_CONFIG, out_signature='as')
+    @dbus_handle_exceptions
+    def getServiceNames(self, sender=None): # pylint: disable=W0613
+        """get service names
+        """
+        log.debug1("config.getServiceNames()")
+        services = [ ]
+        for obj in self.services:
+            services.append(obj.obj.name)
+        return sorted(services)
+
     @dbus_service_method(config.dbus.DBUS_INTERFACE_CONFIG, in_signature='s',
                          out_signature='o')
     @dbus_handle_exceptions
-    def getServiceByName(self, service, sender=None):
+    def getServiceByName(self, service, sender=None): # pylint: disable=W0613
         """object path of service with given name
         """
         service = dbus_to_python(service, str)
@@ -883,16 +953,27 @@ class FirewallDConfig(slip.dbus.service.Object):
 
     @dbus_service_method(config.dbus.DBUS_INTERFACE_CONFIG, out_signature='ao')
     @dbus_handle_exceptions
-    def listZones(self, sender=None):
+    def listZones(self, sender=None): # pylint: disable=W0613
         """list zones objects paths
         """
         log.debug1("config.listZones()")
         return self.zones
 
+    @dbus_service_method(config.dbus.DBUS_INTERFACE_CONFIG, out_signature='as')
+    @dbus_handle_exceptions
+    def getZoneNames(self, sender=None): # pylint: disable=W0613
+        """get zone names
+        """
+        log.debug1("config.getZoneNames()")
+        zones = [ ]
+        for obj in self.zones:
+            zones.append(obj.obj.name)
+        return sorted(zones)
+
     @dbus_service_method(config.dbus.DBUS_INTERFACE_CONFIG, in_signature='s',
                          out_signature='o')
     @dbus_handle_exceptions
-    def getZoneByName(self, zone, sender=None):
+    def getZoneByName(self, zone, sender=None): # pylint: disable=W0613
         """object path of zone with given name
         """
         zone = dbus_to_python(zone, str)
@@ -905,7 +986,7 @@ class FirewallDConfig(slip.dbus.service.Object):
     @dbus_service_method(config.dbus.DBUS_INTERFACE_CONFIG, in_signature='s',
                          out_signature='s')
     @dbus_handle_exceptions
-    def getZoneOfInterface(self, iface, sender=None):
+    def getZoneOfInterface(self, iface, sender=None): # pylint: disable=W0613
         """name of zone the given interface belongs to
         """
         iface = dbus_to_python(iface, str)
@@ -917,13 +998,15 @@ class FirewallDConfig(slip.dbus.service.Object):
         if len(ret) > 1:
             # Even it shouldn't happen, it's actually possible that
             # the same interface is in several zone XML files
-            return " ".join(ret) + "  (ERROR: interface '%s' is in %s zone XML files, can be only in one)" % (iface, len(ret))
+            return " ".join(ret) + \
+                "  (ERROR: interface '%s' is in %s zone XML files, can be only in one)" % \
+                (iface, len(ret))
         return ret[0] if ret else ""
 
     @dbus_service_method(config.dbus.DBUS_INTERFACE_CONFIG, in_signature='s',
                          out_signature='s')
     @dbus_handle_exceptions
-    def getZoneOfSource(self, source, sender=None):
+    def getZoneOfSource(self, source, sender=None): # pylint: disable=W0613
         """name of zone the given source belongs to
         """
         source = dbus_to_python(source, str)
@@ -935,7 +1018,9 @@ class FirewallDConfig(slip.dbus.service.Object):
         if len(ret) > 1:
             # Even it shouldn't happen, it's actually possible that
             # the same source is in several zone XML files
-            return " ".join(ret) + "  (ERROR: source '%s' is in %s zone XML files, can be only in one)" % (source, len(ret))
+            return " ".join(ret) + \
+                "  (ERROR: source '%s' is in %s zone XML files, can be only in one)" % \
+                (source, len(ret))
         return ret[0] if ret else ""
 
     @dbus_service_method(config.dbus.DBUS_INTERFACE_CONFIG,
@@ -969,7 +1054,7 @@ class FirewallDConfig(slip.dbus.service.Object):
     @dbus_service_method(config.dbus.DBUS_INTERFACE_CONFIG_DIRECT,
                          out_signature=Direct.DBUS_SIGNATURE)
     @dbus_handle_exceptions
-    def getSettings(self, sender=None):
+    def getSettings(self, sender=None): # pylint: disable=W0613
         # returns list ipv, table, list of chains
         log.debug1("config.direct.getSettings()")
         return self.config.get_direct().export_config()
@@ -977,7 +1062,7 @@ class FirewallDConfig(slip.dbus.service.Object):
     @dbus_service_method(config.dbus.DBUS_INTERFACE_CONFIG_DIRECT,
                          in_signature=Direct.DBUS_SIGNATURE)
     @dbus_handle_exceptions
-    def update(self, settings, sender=None):
+    def update(self, settings, sender=None): # pylint: disable=W0613
         # returns list ipv, table, list of chains
         log.debug1("config.direct.update()")
         settings = dbus_to_python(settings)
@@ -1033,7 +1118,7 @@ class FirewallDConfig(slip.dbus.service.Object):
     @dbus_service_method(config.dbus.DBUS_INTERFACE_CONFIG_DIRECT,
                          in_signature='sss', out_signature='b')
     @dbus_handle_exceptions
-    def queryChain(self, ipv, table, chain, sender=None):
+    def queryChain(self, ipv, table, chain, sender=None): # pylint: disable=W0613
         ipv = dbus_to_python(ipv)
         table = dbus_to_python(table)
         chain = dbus_to_python(chain)
@@ -1045,7 +1130,7 @@ class FirewallDConfig(slip.dbus.service.Object):
     @dbus_service_method(config.dbus.DBUS_INTERFACE_CONFIG_DIRECT,
                          in_signature='ss', out_signature='as')
     @dbus_handle_exceptions
-    def getChains(self, ipv, table, sender=None):
+    def getChains(self, ipv, table, sender=None): # pylint: disable=W0613
         ipv = dbus_to_python(ipv)
         table = dbus_to_python(table)
         log.debug1("config.direct.getChains('%s', '%s')" % (ipv, table))
@@ -1058,7 +1143,7 @@ class FirewallDConfig(slip.dbus.service.Object):
     @dbus_service_method(config.dbus.DBUS_INTERFACE_CONFIG_DIRECT,
                          in_signature='', out_signature='a(sss)')
     @dbus_handle_exceptions
-    def getAllChains(self, sender=None):
+    def getAllChains(self, sender=None): # pylint: disable=W0613
         log.debug1("config.direct.getAllChains()")
         return self.getSettings()[0]
 
@@ -1067,7 +1152,7 @@ class FirewallDConfig(slip.dbus.service.Object):
     @dbus_service_method(config.dbus.DBUS_INTERFACE_CONFIG_DIRECT,
                          in_signature='sssias')
     @dbus_handle_exceptions
-    def addRule(self, ipv, table, chain, priority, args, sender=None):
+    def addRule(self, ipv, table, chain, priority, args, sender=None): # pylint: disable=R0913
         ipv = dbus_to_python(ipv)
         table = dbus_to_python(table)
         chain = dbus_to_python(chain)
@@ -1088,7 +1173,7 @@ class FirewallDConfig(slip.dbus.service.Object):
     @dbus_service_method(config.dbus.DBUS_INTERFACE_CONFIG_DIRECT,
                          in_signature='sssias')
     @dbus_handle_exceptions
-    def removeRule(self, ipv, table, chain, priority, args, sender=None):
+    def removeRule(self, ipv, table, chain, priority, args, sender=None): # pylint: disable=R0913
         ipv = dbus_to_python(ipv)
         table = dbus_to_python(table)
         chain = dbus_to_python(chain)
@@ -1109,7 +1194,7 @@ class FirewallDConfig(slip.dbus.service.Object):
     @dbus_service_method(config.dbus.DBUS_INTERFACE_CONFIG_DIRECT,
                          in_signature='sssias', out_signature='b')
     @dbus_handle_exceptions
-    def queryRule(self, ipv, table, chain, priority, args, sender=None):
+    def queryRule(self, ipv, table, chain, priority, args, sender=None): # pylint: disable=W0613,R0913
         ipv = dbus_to_python(ipv)
         table = dbus_to_python(table)
         chain = dbus_to_python(chain)
@@ -1127,8 +1212,8 @@ class FirewallDConfig(slip.dbus.service.Object):
         ipv = dbus_to_python(ipv)
         table = dbus_to_python(table)
         chain = dbus_to_python(chain)
-        log.debug1("config.direct.removeRules('%s', '%s', '%s')" %
-                                             (ipv, table, chain, ))
+        log.debug1("config.direct.removeRules('%s', '%s', '%s')" % \
+                   (ipv, table, chain, ))
         self.accessCheck(sender)
         settings = list(self.getSettings())
         settings[1] = []
@@ -1137,7 +1222,7 @@ class FirewallDConfig(slip.dbus.service.Object):
     @dbus_service_method(config.dbus.DBUS_INTERFACE_CONFIG_DIRECT,
                          in_signature='sss', out_signature='a(ias)')
     @dbus_handle_exceptions
-    def getRules(self, ipv, table, chain, sender=None):
+    def getRules(self, ipv, table, chain, sender=None): # pylint: disable=W0613
         ipv = dbus_to_python(ipv)
         table = dbus_to_python(table)
         chain = dbus_to_python(chain)
@@ -1152,7 +1237,7 @@ class FirewallDConfig(slip.dbus.service.Object):
     @dbus_service_method(config.dbus.DBUS_INTERFACE_CONFIG_DIRECT,
                          in_signature='', out_signature='a(sssias)')
     @dbus_handle_exceptions
-    def getAllRules(self, sender=None):
+    def getAllRules(self, sender=None): # pylint: disable=W0613
         log.debug1("config.direct.getAllRules()")
         return self.getSettings()[1]
 
@@ -1196,7 +1281,7 @@ class FirewallDConfig(slip.dbus.service.Object):
     @dbus_service_method(config.dbus.DBUS_INTERFACE_CONFIG_DIRECT,
                          in_signature='sas', out_signature='b')
     @dbus_handle_exceptions
-    def queryPassthrough(self, ipv, args, sender=None):
+    def queryPassthrough(self, ipv, args, sender=None): # pylint: disable=W0613
         ipv = dbus_to_python(ipv)
         args = dbus_to_python(args)
         log.debug1("config.direct.queryPassthrough('%s', '%s')" % \
@@ -1207,7 +1292,7 @@ class FirewallDConfig(slip.dbus.service.Object):
     @dbus_service_method(config.dbus.DBUS_INTERFACE_CONFIG_DIRECT,
                          in_signature='s', out_signature='aas')
     @dbus_handle_exceptions
-    def getPassthroughs(self, ipv, sender=None):
+    def getPassthroughs(self, ipv, sender=None): # pylint: disable=W0613
         ipv = dbus_to_python(ipv)
         log.debug1("config.direct.getPassthroughs('%s')" % (ipv))
         ret = [ ]
@@ -1219,6 +1304,6 @@ class FirewallDConfig(slip.dbus.service.Object):
     @dbus_service_method(config.dbus.DBUS_INTERFACE_CONFIG_DIRECT,
                          out_signature='a(sas)')
     @dbus_handle_exceptions
-    def getAllPassthroughs(self, sender=None):
+    def getAllPassthroughs(self, sender=None): # pylint: disable=W0613
         log.debug1("config.direct.getAllPassthroughs()")
         return self.getSettings()[2]
